""" Propositional submodule.

.. rubric:: Classes

.. autoclass:: PropositionalTaskEncoder
    :members:
.. autoclass:: PropositionalResponseEncoder
    :members:

"""

from .task_encoder_prop import PropositionalTaskEncoder
from .resp_encoder_prop import PropositionalResponseEncoder
