""" CCOBRA response comparator functionality.

.. rubric:: Classes

.. autoclass:: AbsDiffComparator
    :members:
.. autoclass:: EqualityComparator
    :members:
.. autoclass:: NVCComparator
    :members:
.. autoclass:: SquaredDiffComparator
    :members:

"""

from .absdiff import AbsDiffComparator
from .equality import EqualityComparator
from .nvc import NVCComparator
from .squareddiff import SquaredDiffComparator
